% This script simulate a 2D image of a few number of emitters and performs
% the localization of emitters based on continuous basis pursuit.
% 
% Hayato Ikoma, 2015
% Stanford University

addpath('function')
%% Set parameters
N = 16; % pixel number
Delta = 1/N;

sigmaList = [1.18 1.22]/N; % standard deviation of Gaussian kernels
Ix = [7,11]; % value: 0~N-1
Iy = [8,8]; % value: 0~N-1

K = length(Ix); % number of emitters
a0_val = [5000 4000]; % photon counts for each emitters
d0x_val = [.6 -.2]; % shift x: -1~1 close to 1 is difficult.
d0y_val = [.4 -.6]; % shift y

back = 0; % constant background photon

%% Simulate image (convolve positive sources and Gaussian kernels)
a0 = zeros(N,N);
d0x = zeros(N,N);
d0y = zeros(N,N);

for k=1:K
    a0(Iy(k),Ix(k)) = a0_val(k);
    d0x(Iy(k),Ix(k)) = d0x_val(k);
    d0y(Iy(k),Ix(k)) = d0y_val(k);
end

b0x = d0x.*a0;
b0y = d0y.*a0;

gg = (0:N-1)'/N;
[xx,yy] = meshgrid(gg,gg);
xx0 = xx + d0x*Delta/2;
yy0 = yy + d0y*Delta/2;

I = find(a0>0);
x0 = bsxfun(@plus,(0:N-1)/N,d0x*Delta/2);
y0 = bsxfun(@plus,(0:N-1)'/N,d0y*Delta/2);

% observation
im_noiseless = zeros(N,N);
phi  = @(x,y,sigma)exp(-(x.^2+y.^2)/(2*sigma^2));
for k = 1:K
    sigma = sigmaList(k);
    % spike position in 0~1 2D grid
    sp_posx = (Ix(k)-1)/N + d0x_val(k)*Delta/2; 
    sp_posy = (Iy(k)-1)/N + d0y_val(k)*Delta/2;
    xx_temp = xx - sp_posx;
    yy_temp = yy - sp_posy;
    
    xx_temp = mod(xx_temp,1); xx_temp(xx_temp>.5) = xx_temp(xx_temp>.5)-1;
    yy_temp = mod(yy_temp,1); yy_temp(yy_temp>.5) = yy_temp(yy_temp>.5)-1;
    psf = phi(xx_temp,yy_temp,sigma);
    
    psf = round(a0_val(k)*psf/(sum(psf(:)))); % quantize
    im_noiseless = im_noiseless + psf;
end

% Add Poisson noise
im_noisy = poissrnd(im_noiseless + back);

% visualize a simulated image
figure;
imshow(im_noisy,[])

%% Localization
im_noisy = im_noisy/max(im_noisy(:));
sigmaList = [1.2]/N; % standard deviation used for estimation

alpha = 2; % upsampling factor. 2 should be fine.


[estLoc,hist] = upsampleCBPCSC2D_local(im_noisy,alpha,sigmaList);


%% Visualize the result
P = alpha*N; 
c = estLoc.c;
loc_x = estLoc.loc_x;
loc_y = estLoc.loc_y;

clf;
subplot(1,2,1);
imshow(im_noisy,[]);
title('Observation')

subplot(1,2,2);
hold on;
scatter(x0(I),y0(I),'.r')
scatter(loc_x,loc_y,'.b');
set(gca,'xtick',(0:N-1)/N+Delta/2,'xticklabel',[]);
set(gca,'ytick',(0:N-1)/N+Delta/2,'yticklabel',[]);
xlabel('x'); ylabel('y');
xlim([0-Delta/2 1-Delta/2]);
ylim([0-Delta/2 1-Delta/2]);
axis square;
ax = gca;
ax.YDir = 'reverse';
grid on;
box on;
title('Localization')
set(gcf,'color','w');
legend('Ground truth', 'Estimation')
