function [estLoc,hist] = upsampleCBPCSC1D(b,alpha,sigmaList)
% b: observed image (image size has to be square.)
% sigmaList: list of std of gaussian kernels
% alpha: upsampling factor
%% parameter for ADMM
lambda = .1; rho = 2; maxIter1 = 500; maxIter2 = 500;

%% Construct kernels
num_kernel = length(sigmaList);

N = length(b); % number of sample
P = alpha*N; % upsampled points

Delta = 1/P;
t = [0:P/2, -P/2+1:-1]' / P;

upsample = @(x)upsampling(x,1,alpha);
downsample = @(y)y(1:alpha:end);

phi = @(t,sigma)exp(-t.^2/(2*sigma^2));
phi1 = @(t,sigma)-t/(sigma^2).*exp(-t.^2/(2*sigma^2));

convol = @(x,H) real(ifft(sum(fft(x,[],1).*H,2)));

psf = zeros(length(t),num_kernel);
psf1 = zeros(length(t),num_kernel);
for ii = 1:num_kernel
    psf(:,ii) = phi(t,sigmaList(ii));
    psf1(:,ii) = phi1(t,sigmaList(ii));
end


H = fft(psf,[],1);
dH = -fft(psf1*Delta/2,[],1);
HH = [H,dH];
Gamma = @(x) convol(x,HH);


C  = @(u)u(:,1+num_kernel:end)+1i*u(:,1:num_kernel);
Ci = @(v)[imag(v), real(v)];
ProjOct = @(v)max(real(v),0) + 1i*max(imag(v),0);
ProjS = @(u)Ci(exp(1i*pi/4)*ProjOct(exp(-1i*pi/4)*C(u)));

%% Deconvolution 1
HscInv = bsxfun(@times,conj(HH),1./(1+sum(abs(HH).^2,2)));

temp1 = ones(P,1);
temp2 = upsample(downsample(ones(P,1)));
invStS = 1./(temp1 + 1/rho * temp2); % (I + 1/rho * S'S)^-1
F = @(x)1/2*norm(b- downsample(Gamma(x)), 'fro')^2;
updatex = @(u1,u2)update(u1,u2,HH,HscInv);
ProxF = @(v)invStS.*(v + 1/rho * upsample(b));
ProxG1 = @(w,lambda)ProjS( w-[lambda*ones(size(w,1),num_kernel), zeros(size(w,1),num_kernel)] );

% the primary functional
E = @(u)F(u) + lambda*norm(u(:,1:num_kernel),1);

eta1 = zeros(P,1);
eta2 = zeros(P,num_kernel*2);
u1 = zeros(P,1);
u2 = zeros(P,num_kernel*2);

hist.r1_1 = zeros(maxIter1,1);
hist.r2_1 = zeros(maxIter1,1);
hist.E_1 = zeros(maxIter1,1);

for ii=1:maxIter1
    x = updatex(u1+eta1,u2+eta2);
    
    Gamma_x = Gamma(x);
    u1_new = ProxF( Gamma_x - eta1 );
    u2_new = ProxG1( x - eta2, lambda/rho);
    eta1 = eta1 + u1_new - Gamma_x;
    eta2 = eta2 + u2_new - x;
    
    u1 = u1_new;
    u2 = u2_new;
    
    hist.r1_1(ii) = norm(u1 - Gamma_x,'fro');
    hist.r2_1(ii) = norm(u2 - x,'fro');    
    
    hist.E(ii) = E(x);
end

x(x<1e-1) = 0;

%% Deconvolution 2
C = find(x == 0); % support
% ProxG2 = @(x) ProjC(x,C);
ProxG2 = @(x)ProjS( ProjC(x,C) );

eta1 = zeros(P,1);
eta2 = zeros(P,num_kernel*2);
u1 = zeros(P,1);
u2 = zeros(P,num_kernel*2);

hist.r1_2 = zeros(maxIter2,1);
hist.r2_2 = zeros(maxIter2,1);
hist.E_2 = zeros(maxIter2,1);

for ii=1:maxIter2
    x = updatex(u1+eta1,u2+eta2);
    
    Gamma_x = Gamma(x);
    u1_new = ProxF( Gamma_x - eta1 );
    u2_new = ProxG2( x - eta2 );
    eta1 = eta1 + u1_new - Gamma_x;
    eta2 = eta2 + u2_new - x;
    
    u1 = u1_new;
    u2 = u2_new;
    
    hist.r1_2(ii) = norm(u1 - Gamma_x,'fro');
    hist.r2_2(ii) = norm(u2 - x,'fro');    
    
    hist.E_2(ii) = E(x);
end

% x = u2;
x(x<1e-1) = 0;

%% Compute center mass
a = x(:,1:num_kernel);
b = x(:,num_kernel+1:end);

delta = Delta/2 * b./a;
tt = (0:P-1)/P;

bw = logical(a);

estLoc.c = [];
estLoc.loc_x = [];

k = 1;
for l = 1:num_kernel
    ii = 1;
    while ii <= size(a,1)
        if bw(ii,l)==0
            ii = ii + 1;            
        else
            c = 0; temp_x = 0;
            while bw(ii,l)>0
                c = c + a(ii,l);
                temp_x = temp_x + a(ii,l)*(tt(ii)+delta(ii,l));
                ii = ii+1;                
            end
            estLoc.c(k) = c;
            estLoc.loc_x(k) = temp_x/c;

            k = k + 1;
        end
    end
end

end

function x_new = update(u1,u2,HH,HscInv)
    U1 = fft(u1,[],1);
    U2 = fft(u2,[],1);
    cHU = bsxfun(@times,conj(HH),U1);
    Z = cHU + U2;
    temp = sum(HH.*Z,2);
    x_new = real(ifft(Z - bsxfun(@times,HscInv,temp),[],1));
end

function w = ProjC(w,C)           
    w(C) = 0;
end

function y = upsampling(x,d,p)
    % upsampling - add p zeros between samples along dimension d
    %
    %   y = upsampling(x,d,p);
    %
    %   default is p==2, d==1
    %
    %   Copyright (c) 2009 Gabriel Peyre

    if nargin<3
        p = 2;
    end
    if nargin<2
        d = 1;
    end

    switch d
        case 1
            y = zeros(p*size(x,1),size(x,2),size(x,3)); 
            y(1:p:end,:,:) = x;
        case 2
            y = zeros(size(x,1),p*size(x,2),size(x,3)); 
            y(:,1:p:end,:) = x;
        case 3
            y = zeros(size(x,1),size(x,2),p*size(x,3)); 
            y(:,:,1:p:end) = x;
        otherwise
            error('Not implemented');
    end
end

