function [estLoc,hist] = upsampleCBPCSC2D(b,alpha,sigmaList)
% b: observed image (image size has to be square.)
% sigmaList: list of std of gaussian kernels
% alpha: upsampling factor
%% parameter for ADMM
lambda = .1; rho = 2; maxIter = 500;

%% background subtraction
% background = background_estimation(b,1,6,'db6',10);
% b = b- background;

%% construct filters
N = size(b,1); % size of an input image
P = alpha*N; % upsampled points
Delta = 1/P; % distance between sampling grid

num_kernel = length(sigmaList);
[H,Hdx,Hdy] = constructGaussians(sigmaList,P);

%% Set some useful function handles
HH = cat(3,H,-Hdx*Delta/2,-Hdy*Delta/2);
convol = @(x,H) real(ifft2( sum(fft2(x).*H,3) ));
Gamma = @(x) convol(x,HH);
upsample = @(x)upsampling(upsampling(x,1,alpha),2,alpha);
downsample = @(y)y(1:alpha:end,1:alpha:end);

F = @(x) 1/2*norm( b - downsample(Gamma(x)), 'fro')^2;

%% Define the proximal operators for Deconvolution 1 
temp1 = ones(P,P);
temp2 = upsample(downsample(ones(P,P,1)));
invStS = 1./(temp1 + 1/rho * temp2); % (I + 1/rho * S'S)^-1

HscInv = bsxfun(@times,conj(HH),1./(1+sum(abs(HH).^2,3)));

updatex = @(u1,u2)update(u1,u2,HH,HscInv);
ProxF = @(v)invStS.*(v + 1/rho * upsample(b));

S = cat(3,lambda*ones(P,P,num_kernel),zeros(P,P,2*num_kernel));
ProxG1 = @(w,lambda) projectOnCone( w - S ,num_kernel);

%% Deconvolve 1
display('Deconvolution 1 starting...')
E = @(u)F(u) + lambda*sum(sum(sum(u(:,:,1:num_kernel))));

eta1 = zeros(P,P);
eta2 = zeros(P,P,num_kernel*3);
u1 = zeros(P,P);
u2 = zeros(P,P,num_kernel*3);

hist.r1 = zeros(maxIter,1);
hist.E = zeros(maxIter,1);

for ii=1:maxIter
    if mod(ii,1000)==0
        display(['iteration: ' num2str(ii)])
    end
    
    x = updatex(u1+eta1,u2+eta2);
    Gamma_x = Gamma(x);
    
    u1_new = ProxF( Gamma_x - eta1 );
    u2_new = ProxG1( x - eta2, lambda/rho);
    eta1 = eta1 + u1_new - Gamma_x;
    eta2 = eta2 + u2_new - x;
    
    u1 = u1_new;
    u2 = u2_new;
    
    hist.r1(ii) = norm(u1 - Gamma_x,'fro');
    
    hist.E(ii) = E(x);

end

x = u2;
x(x<1e-5) = 0;

%% Deconvolve 2
display('Deconvolution 2 starting...')
C = x == 0; % support set
ProxG2 = @(x) ProjC(x,C,num_kernel);

eta1 = zeros(P,P);
eta2 = zeros(P,P,num_kernel*3);
u1 = zeros(P,P);
u2 = zeros(P,P,num_kernel*3);

for ii=1:maxIter
    if mod(ii,1000)==0
        display(['iteration: ' num2str(ii)])
    end
    
    x = updatex(u1+eta1,u2+eta2);
    
    Gamma_x = Gamma(x);
    u1_new = ProxF( Gamma_x - eta1 );
    u2_new = ProxG2( x - eta2 );
    eta1 = eta1 + u1_new - Gamma_x;
    eta2 = eta2 + u2_new - x;
    
    u1 = u1_new;
    u2 = u2_new;
    
    hist.r1(ii) = norm(u1 - Gamma_x,'fro');   
    
    hist.E(ii) = E(x);
end

% x = u2;
x(x<1e-1) = 0;

%% Compute center mass
a = x(:,:,1:num_kernel);
bx = x(:,:,num_kernel+1:2*num_kernel);
by = x(:,:,2*num_kernel+1:end);

delta_x = Delta/2 * bx./a;
delta_y = Delta/2 * by./a;

[xx,yy] = meshgrid((0:P-1)/P,(0:P-1)/P);

% find the support of a
bw = logical(a);
L = bwlabel(bw,8); % 8-connected region

% find the center mass of all connected regions
estLoc.c = [];
estLoc.loc_x = [];
estLoc.loc_y = [];
k = 1;
for ii = 1:max(L(:))
    id = find(L==ii);
    
    if length(id)>4 % reject estimated locations that has more than 4-connected pixels.
        break;
    end
    
    c = sum(a(id));
    estLoc.c(k) = c; % intensity (mass)
    estLoc.loc_x(k) = sum(a(id).*(xx(id)+delta_x(id)))/c;
    estLoc.loc_y(k) = sum(a(id).*(yy(id)+delta_y(id)))/c;
    k = k + 1;
end

end

function x_new = update(u1,u2,HH,HscInv)
    U1 = fft2(u1);
    U2 = fft2(u2);
    cHU = bsxfun(@times,conj(HH),U1);
    
    Z = cHU + U2;
    temp = sum(HH.*Z,3);
    x_new = real(ifft2(Z - bsxfun(@times,HscInv,temp)));
end

function w = ProjC(w,C,num_kernel)   
% Projection operator
    w(C) = 0;
    w = projectOnCone(w,num_kernel);
end

function [H,Hdx,Hdy] = constructGaussians(sigmaList,N)
    phi  = @(x,y,sigma)exp(-(x.^2+y.^2)/(2*sigma^2));
    phi_dx = @(x,y,sigma)-x/(sigma^2).*exp(-(x.^2+y.^2)/(2*sigma^2));
    phi_dy = @(x,y,sigma)-y/(sigma^2).*exp(-(x.^2+y.^2)/(2*sigma^2));
    
    t = [0:N/2, -N/2+1:-1]'/N; 
    [xx, yy] = meshgrid(t,t);

    num_kernels = length(sigmaList);
    H = zeros(N,N,num_kernels);
    Hdx = zeros(N,N,num_kernels);
    Hdy = zeros(N,N,num_kernels);
    for ii = 1:num_kernels
        sigma = sigmaList(ii);
        h = phi(xx,yy,sigma);
        h_dx = phi_dx(xx,yy,sigma);
        h_dy = phi_dy(xx,yy,sigma);
        H(:,:,ii) = fft2(h);
        Hdx(:,:,ii) = fft2(h_dx);
        Hdy(:,:,ii) = fft2(h_dy);
    end

end

function v_out = projectOnCone(input_u,num_kernel)
% projection operators on the convex cone

    [N,M,K] = size(input_u);
    u = reshape(input_u,N*M*num_kernel,[]);

    Cx  = @(u)u(:,2)+1i*u(:,1);
    Cy = @(u)u(:,3)+1i*u(:,1);
    Ci = @(v)[imag(v), real(v)];
    reorder = @(u) [u(:,1),u(:,3),u(:,2)];
    ProjOct = @(v)max(real(v),0) + 1i*max(imag(v),0);
    ProjCx = @(u) [Ci(exp(1i*pi/4)*ProjOct(exp(-1i*pi/4)*Cx(u))),u(:,3)];
    ProjCy = @(u) reorder([Ci(exp(1i*pi/4)*ProjOct(exp(-1i*pi/4)*Cy(u))),u(:,2)]);

    vx = ProjCx(u);
    vy = ProjCy(u);

    p = 1/sqrt(3)*[1, 1, 1];

    Projp = @(u) bsxfun(@times,...
        sum(bsxfun(@times,[u(:,1),abs(u(:,2)),abs(u(:,3))],p),2), ...
        1/sqrt(3)*[ones(size(u,1),1), sign(u(:,2)), sign(u(:,3))]);


    vp = Projp(u);

    v = u;

    verify = @(u) (abs(u(:,3)) <= abs(u(:,1))) .* (abs(u(:,2)) <= abs(u(:,1))) .* (u(:,1)>=0);

    vxid = verify(vx);
    vyid = verify(vy);
    vpid = verify(vp).*~vxid.*~vyid;

    zeroid = (abs(u(:,2)+u(:,3))<= abs(u(:,1))) .* (u(:,1)<0);

    z = 0*u;
    v(find(vxid),:) = vx(find(vxid),:);
    v(find(vyid),:) = vy(find(vyid),:);
    v(find(zeroid),:) = z(find(zeroid),:);
    v(find(vpid),:) = vp(find(vpid),:);

    v_out = reshape(v,[N M K]);
end

function y = upsampling(x,d,p)
% upsampling - add p zeros between samples along dimension d
%
%   y = upsampling(x,d,p);
%
%   default is p==2, d==1
%
%   Copyright (c) 2009 Gabriel Peyre

    if nargin<3
        p = 2;
    end
    if nargin<2
        d = 1;
    end

    switch d
        case 1
            y = zeros(p*size(x,1),size(x,2),size(x,3)); 
            y(1:p:end,:,:) = x;
        case 2
            y = zeros(size(x,1),p*size(x,2),size(x,3)); 
            y(:,1:p:end,:) = x;
        case 3
            y = zeros(size(x,1),size(x,2),p*size(x,3)); 
            y(:,:,1:p:end) = x;
        otherwise
            error('Not implemented');
    end
end